import discord
from discord.ext import commands
from seija.embeds import oldembeds as osuembed
from seija.modules import permissions
from seija.reusables import exceptions
from seija.embeds import newembeds as osuwebembed


class Osu(commands.Cog):
    def __init__(self, bot):
        self.bot = bot

    @commands.command(name="mapset", brief="Show mapset info")
    @commands.check(permissions.is_not_ignored)
    async def mapset(self, ctx, mapset_id: str):
        result = await self.bot.osu.get_beatmapset(s=mapset_id)

        embed = await osuembed.beatmapset(result)
        if not embed:
            await ctx.send(content="`No mapset found with that ID`")
            return

        await ctx.send(embed=embed)

    @commands.command(name="user", brief="Show osu user info")
    @commands.check(permissions.is_not_ignored)
    async def user(self, ctx, *, username):
        result = await self.bot.osuweb.get_user_array(username)

        if not result:
            await ctx.send(content="`No user found with that username`")
            return

        embed = await osuwebembed.user_array(result)

        await ctx.send(embed=embed)

    @commands.command(name="ts", brief="Send an osu editor clickable timestamp")
    @commands.guild_only()
    @commands.check(permissions.is_not_ignored)
    async def ts(self, ctx, *, string):
        """
        Send an osu editor clickable timestamp
        Must start with a timestamp
        This command needs a proper solution, this is half-assed.
        """

        if "-" in string:
            timestamp_data = string.split("-")
            timestamp_link = (timestamp_data[0]).strip().replace(" ", "_")
            try:
                timestamp_desc = "- " + timestamp_data[1]
            except IndexError:
                timestamp_desc = ""
        else:
            timestamp_link = string.strip().replace(" ", "_")
            timestamp_desc = ""
        embed = discord.Embed(
            description=f"<osu://edit/{timestamp_link}> {timestamp_desc}",
            color=ctx.author.colour
        )
        embed.set_author(
            name=ctx.author.display_name,
            icon_url=ctx.author.display_avatar.url
        )
        await ctx.send(embed=embed)

        try:
            await ctx.message.delete()
        except discord.Forbidden as e:
            print(f"deletion of message {ctx.channel.id}/{ctx.message.id} failed")
            print(e)


async def setup(bot):
    await bot.add_cog(Osu(bot))
